package pp.model;

import java.io.IOException;
import java.io.InputStream;
import java.util.Hashtable;
import javax.microedition.lcdui.Image;

import pp.model.event.PPTrigger;

public class LevelGenerator {

	//Constants::
	public static final String LEVEL_1_NAME="/pp/data/lvl1.lvl";//Name of level 1
	public static final String LEVEL_1_SCRIPT_NAME="/pp/data/lvl1.slvl";//Name of the script file of level 1
	public static final int LEVEL_BLOCK_WIDTHxHEIGHT=32;//Standard width and height of all objects in the game world
	public static final char SCRIPT_MOVEMENT='m';
	public static final char SCRIPT_VERTICAL='v';
	public static final char SCRIPT_TRIGGER='t';
	public static final char SCRIPT_COLLISION='c';
	public static final char SCRIPT_IMAGE='i';
	
	//Variables::
	private Image IMAGE_BLOCK_1=null;//block1.png
	private Image IMAGE_BLOCK_2=null;
	private Image IMAGE_BLOCK_3=null;
	private Image IMAGE_TRIGGER_LEFT=null;
	private Image IMAGE_TRIGGER_RIGHT=null;
	private Image IMAGE_TRANSPARENT=null;
	private Image IMAGE_EXIT=null;
	
	//Constructor::
	public LevelGenerator(){
		try {
			IMAGE_BLOCK_1=Image.createImage("/pp/images/block1.png");
			IMAGE_BLOCK_2=Image.createImage("/pp/images/block2.png");
			IMAGE_BLOCK_3=Image.createImage("/pp/images/block3.png");
			IMAGE_TRIGGER_LEFT=Image.createImage("/pp/images/trigger_l_1.png");
			IMAGE_TRIGGER_RIGHT=Image.createImage("/pp/images/trigger_r_1.png");
			IMAGE_TRANSPARENT=Image.createImage("/pp/images/trans.png");
			IMAGE_EXIT=Image.createImage("/pp/images/exit2.png");
			
		} catch (IOException e) {
			e.printStackTrace();
		}				
	}
	
	//Reads a level from a file::
	public Level readLevel(String levelName,String scriptName,PPTrigger trigger){
		InputStream is = getClass().getResourceAsStream("/pp/data/"+levelName);//The level file
		InputStream isScript=getClass().getResourceAsStream("/pp/data/"+scriptName);//The script file that is related to the level file
		try 
	    {
	    	//Temp level, returned if the level creation is succesful::
	    	Level lvl=new Level(trigger);	    	
			//File Buffers::
			StringBuffer sb = new StringBuffer();
			StringBuffer sbScript=new StringBuffer();
			int chr, i = 0,chrT;
			/*Property hashmap, stores properties as <char c,ObjectProperty objProp> 
			 * where the char is the identifier of that specific property which again
			 * is tied with the lvl file and thus acts a general object proeprty that 
			 * can be used of any object in the lvl that uses the char as identifier.
			 */
			Hashtable propMap=new Hashtable();			
			  //////////////////////////////////////////////////////////////
			//::::::::::::::::::::|SCRIPT FILE|::::::::::::::::::::::::://	 
           //////////////////////////////////////////////////////////////
			//Read the script file for the level;;
			while( ( chrT =  isScript.read()) != -1 ){
				sbScript.append((char)chrT);
			}
			//Goes through the script buffer creating triggers and properties::
			i=0;
			while(i<sbScript.length()){
				char c=sbScript.charAt(i);
				//Investigates 'c' and finds out whether to create a new object property::
				if(c == Level.LEVEL_CONST_INVISIBLE_SPACE_THINGIE){
					//doNothing();
				}else if(c == Level.LEVEL_CONST_SPACE){
					//doNothing();
				}else if(c==Level.LEVEL_CONST_LINE_BREAK){
					//doNothing();
				}
				else{
					/*
					 * The object properties are on the form described below::
					 * <objID>;m:<0/1>,<x-speed>:<y-speed>,<movement speed>,<event-id>;v:<0/1>;t:<0/1>,<trigger id>;c:<0/1>;|
					 * Each property has an id. A object can be movable, which is assigned 'm'.
					 * The paramters of the movement then follows if the object is movable. The object 
					 * can also allow vertical movement, denoted with 'v'. The object is allowed to have a trigger associated
					 * with it that can trigger movement of other objects, 't', and the trigger has
					 * an ID associated with it. The object may also decide whether to have collisions or not.
					 */
					ObjectProperty objProp=new ObjectProperty();//The properties of the object
					Character ch=new Character(c);//Key to the final objectproperty
					//Find all property values::
					i+=2;
					while((c=sbScript.charAt(i))!='|'){
						//MOVEMENT HANDLING::
						if(c==SCRIPT_MOVEMENT){
							i+=2;
							c=sbScript.charAt(i);
							//The object is not movable if the value is '0', and is movable otherwise::
							if(c=='0'){
								objProp.setMovable(false);
								i++;
							}else{
								objProp.setMovable(true);
								i+=2;
								String t1="";
								int xx=0,yy=0;
								
								//Finds the X-displacement::
								while((c=sbScript.charAt(i))!=':'){
									t1+=c;	
									i++;
								}
								xx=Integer.parseInt(t1);
								objProp.setMX(xx);
								i++;
								t1="";
								
								//Finds the Y-displacement::
								while((c=sbScript.charAt(i))!=','){
									t1+=c;
									i++;
								}
								yy=Integer.parseInt(t1);
								objProp.setMY(yy);
								i++;
								
								//Finds the movement speed::
								int speed=0;
								t1="";
								while((c=sbScript.charAt(i))!=','){
									t1+=c;
									i++;
								}
								speed=Integer.parseInt(t1);
								objProp.setMSpeed(speed);
								i++;
								
								//Finds the event-trigger that triggers the movement::
								int eID=0;
								t1="";
								while((c=sbScript.charAt(i))!=';'){
									t1+=c;
									i++;
								}
								eID=Integer.parseInt(t1);
								objProp.setMovevementEventID(eID);
							}
						}
						//VERTICAL MOVEMENT HANDLING::
						if(c==SCRIPT_VERTICAL){
							i+=2;
							c=sbScript.charAt(i);
							int vert=0;
							String t1="";
							//Finds the vertical movement boolean::
							while((c=sbScript.charAt(i))!=';'){
								t1+=c;
								i++;
							}
							vert=Integer.parseInt(t1);
							//Checks to see whether the object should have vertical movement or not::
							if(vert==0){
								objProp.setVerticalCharacterMovable(false);
							}else{
								objProp.setVerticalCharacterMovable(true);
							}
						}
						//TRIGGER HANDLING::
						if(c==SCRIPT_TRIGGER){
							i+=2;
							c=sbScript.charAt(i);
							int trig=0;
							String t1="";
							//Finds the trigger boolean::
							while((c=sbScript.charAt(i))!=','){
								t1+=c;
								i++;
							}
							trig=Integer.parseInt(t1);
							//Checks to see whether the object should have collision detection or not::
							if(trig==0){
								objProp.setHasTrigger(false);
								i++;
							}else{
								objProp.setHasTrigger(true);
								int tID=0;
								String t2="";
								i++;
								//Finds the trigger id::
								while((c=sbScript.charAt(i))!=';'){
									t2+=c;
									i++;
								}
								tID=Integer.parseInt(t2);								
								objProp.setTriggerID(tID);
							}
						}
						//COLLISION HANDLING::
						if(c==SCRIPT_COLLISION){
							i+=2;
							c=sbScript.charAt(i);
							int col=0;
							String t1="";
							//Finds the collision boolean::
							while((c=sbScript.charAt(i))!=';'){
								t1+=c;
								i++;
							}
							col=Integer.parseInt(t1);
							//Checks to see whether the object should have a collision or not::
							if(col==0){
								objProp.setHasCollision(false);
							}else{
								objProp.setHasCollision(true);
							}
						}
						//IMAGE HANDLING::
						if(c==SCRIPT_IMAGE){
							i+=2;
							c=sbScript.charAt(i);
							int img=0;
							String t1="";
							while((c=sbScript.charAt(i))!=','){
								t1+=c;
								i++;
							}
							img=Integer.parseInt(t1);
							//Checks to see whether the object has an image associated with it::
							if(img==0){
								objProp.setHasImage(false);
								i++;
							}else{
								objProp.setHasImage(true);
								int imgID=0;
								String t2="";
								i++;
								//Finds the image id of the object
								while((c=sbScript.charAt(i))!=';'){
									t2+=c;
									i++;
								}
								imgID=Integer.parseInt(t2);								
								objProp.setImageID(imgID);
							}
						}
						i++;
					}
					//Add property to hashtable with 'ch' as the key::
					propMap.put(ch, objProp);
				}
				i++;
			}
             //////////////////////////////////////////////////////////////
			//::::::::::::::::::::|END SCRIPT FILE|::::::::::::::::::::://
           //////////////////////////////////////////////////////////////
			
			 //////////////////////////////////////////////////////////////
			//::::::::::::::::::::|LEVEL FILE|:::::::::::::::::::::::::://
		   //////////////////////////////////////////////////////////////	
			// Read the level information until the end of the stream::      
			while ((chr = is.read()) != -1){
			    sb.append((char) chr);   
			}
			
			//Goes through the level buffer creating level objects::
			i=0;		
			//Reads the rest of the level::
			int XX=0,YY=0;//hBlocks, use to calculate x and y position of the objects;
			while(i<sb.length()){
				char c=sb.charAt(i);
				//Checks each char to see whether a object should be created::
				//SPACE,white block::
				if(c==Level.LEVEL_CONST_SPACE){
					//System.out.println(" :: STUFF!!!!!!");
					XX++;
				}
				//'\r', do nothing::
				else if(c==Level.LEVEL_CONST_INVISIBLE_SPACE_THINGIE){
					//System.out.println(" :: STUFF 2 !!!!!!");
				}
				//LINE-BREAK, update variables::
				else if(c==Level.LEVEL_CONST_LINE_BREAK){
					XX=0;//Set the x position to 0 since a new row of the map has started
					YY++;//Updates the  position since a new row has started
				}
				//Create a start object::
				else if(c==Level.LEVEL_CONST_START_BLOCK){
					ObjectProperty objProp2=new ObjectProperty();
					objProp2.setHasCollision(false);
					objProp2.setHasImage(false);
					objProp2.setHasTrigger(false);
					objProp2.setMovable(false);
					objProp2.setVerticalCharacterMovable(false);
					
					Vector2D v=new Vector2D(XX*LevelGenerator.LEVEL_BLOCK_WIDTHxHEIGHT,YY*LevelGenerator.LEVEL_BLOCK_WIDTHxHEIGHT);
					StartPPObject start=new StartPPObject(v,IMAGE_TRANSPARENT,objProp2);
					lvl.addLvlObject(start);
					XX++;
				}
				//Create exit block::
				else if(c==Level.LEVEL_CONST_END_BLOCK){
					ObjectProperty objProp2=new ObjectProperty();
					objProp2.setHasCollision(false);
					objProp2.setHasImage(true);
					objProp2.setHasTrigger(false);
					objProp2.setMovable(false);
					objProp2.setVerticalCharacterMovable(false);
					
					Vector2D v=new Vector2D(XX*LevelGenerator.LEVEL_BLOCK_WIDTHxHEIGHT,YY*LevelGenerator.LEVEL_BLOCK_WIDTHxHEIGHT);
					ExitPPObject exit=new ExitPPObject(v,IMAGE_EXIT,objProp2);
					lvl.addExitObject(exit);
					lvl.addLvlObject(exit);
					XX++;
				}
				//Create an object::
				//TODO:: find properties and add them correctly
				else{
					//Checks to see whether the properties list contains the object with key 'c'::
					if(propMap.containsKey(new Character(c))){
						ObjectProperty objProp=new ObjectProperty();
						ObjectProperty objPropT=((ObjectProperty)propMap.get(new Character(c)));
						//objProp=objPropT;
						objProp.setVariablesFromObjectProperty(objPropT);
						ImgPPObject imgObj=null;
						//Sets the appropriate image to the image object according to the image id::
						if(objProp.getImageID()==1){
							Vector2D v=new Vector2D(XX*LevelGenerator.LEVEL_BLOCK_WIDTHxHEIGHT,YY*LevelGenerator.LEVEL_BLOCK_WIDTHxHEIGHT);
							imgObj=new ImgPPObject(v,IMAGE_BLOCK_1,objProp);
						}else if(objProp.getImageID()==2){
							Vector2D v=new Vector2D(XX*LevelGenerator.LEVEL_BLOCK_WIDTHxHEIGHT,YY*LevelGenerator.LEVEL_BLOCK_WIDTHxHEIGHT);
							imgObj=new ImgPPObject(v,IMAGE_BLOCK_2,objProp);
						}else if(objProp.getImageID()==3){
							Vector2D v=new Vector2D(XX*LevelGenerator.LEVEL_BLOCK_WIDTHxHEIGHT,YY*LevelGenerator.LEVEL_BLOCK_WIDTHxHEIGHT);
							imgObj=new ImgPPObject(v,IMAGE_BLOCK_3,objProp);
						}else if(objProp.getImageID()==4){
							Vector2D v=new Vector2D(XX*LevelGenerator.LEVEL_BLOCK_WIDTHxHEIGHT,YY*LevelGenerator.LEVEL_BLOCK_WIDTHxHEIGHT);
							imgObj=new ImgPPObject(v,IMAGE_TRIGGER_LEFT,objProp);
						}else if(objProp.getImageID()==5){
							Vector2D v=new Vector2D(XX*LevelGenerator.LEVEL_BLOCK_WIDTHxHEIGHT,YY*LevelGenerator.LEVEL_BLOCK_WIDTHxHEIGHT);
							imgObj=new ImgPPObject(v,IMAGE_TRIGGER_RIGHT,objProp);
						}
						if(objProp==null)System.out.println("sfgdfg");
						//Checks to see whether the object is a trigger object::
						if(objProp.isHasTrigger()){
							lvl.addTriggerObject(imgObj);
						}
						//Checks to see whether the object is a movable object::
						if(objProp.isMovable()){
							trigger.addPPEventListener(imgObj);
							lvl.addMovableObject(imgObj);
						}
						//Finally adds the object to the general level object list::	
						lvl.addLvlObject(imgObj);	
					}
					XX++;
				}
				i++;
			}
			 //////////////////////////////////////////////////////////////
			//::::::::::::::::::::|END LEVEL FILE|:::::::::::::::::::::://
           //////////////////////////////////////////////////////////////          			  
			
			//Close the input streams::
			is.close();
			isScript.close();
			
			return lvl;
	    }catch (Exception e){  
	    	e.printStackTrace();
		}
	    return null;
	}
}
