package pp.model;

import java.io.IOException;

import javax.microedition.lcdui.Image;

public class Player {
	
	//Variables::
	private Vector2D vPos;//the position of the player in the game world
	private Vector2D vSpeed;//the speed of the player
	private int walkingSpeed=50;
	private int jumpingSpeed=350;
	private PlayerSprite pSprite;//the player image sprite
	private Club club;
	private boolean jumping;
	private boolean falling;
	private boolean clubbed;
	private boolean right;//shows which direction the player is moving in, or was previously walking in while standing still
	private int ID;
	
	//Constants::
	private final int PLAYER_MAX_WALKING_SPEED=4*walkingSpeed;
	
	//Constructor::
	public Player(Vector2D v,int id) throws IOException{
		vPos=v;
		vSpeed=new Vector2D(0,0);
		Image image = Image.createImage(PlayerSprite.PLAYER_SPRITE_NAME);//Player image
		Image image2 = Image.createImage(Club.CLUB_SPRITE_NAME);//Club image
		pSprite=new PlayerSprite(image,vPos.x,vPos.y);
		jumping=false;
		falling=false;
		club=new Club(this,image2);
		ID=id;
	}
	
	//Sets the position of the player sprite::
	public void updateSpritePos(){
		pSprite.setPosition(vPos.x, vPos.y);
		club.updateClubPosition();
	}
	//Sets the speed vector of the player to the left::
	public void moveLeft(int i){
		vSpeed.x-=i*walkingSpeed;
		right=false;
		if(vSpeed.x < -PLAYER_MAX_WALKING_SPEED)vSpeed.x=-PLAYER_MAX_WALKING_SPEED;
		//screenX+=walkingSpeed;
	}
	//Sets the speed vector of the player to the right::
	public void moveRight(int i){
		vSpeed.x+=i*walkingSpeed;
		right=true;
		if(vSpeed.x>PLAYER_MAX_WALKING_SPEED)vSpeed.x=PLAYER_MAX_WALKING_SPEED;
	}
	//Sets the speed vector of the player to accomodate jumping::
	public void jump(){
		if(!jumping && !clubbed && !falling){
			vSpeed.y-=jumpingSpeed;
			jumping=true;
		}
	}
	//Sets the speed vector to accomodate the club mechanic::
	public void clubMe(boolean right){
		if(!clubbed){
			clubbed=true;
			vSpeed.y-=jumpingSpeed;
			if(right)vSpeed.x+=(2*PLAYER_MAX_WALKING_SPEED);
			else vSpeed.x-=(2*PLAYER_MAX_WALKING_SPEED);
		}
	}
	//Checks to see whether the player 'p' collides with the club of this player::
	public boolean clubPlayerCollision(Player p){
		//Only check for collisions if the club is actively used by the player::
		if(club.isClubbing()){
			return club.collidesWith(p.getPSprite(), true);
		}else{
			return false;
		}
	}	
	
	/*
	 * Getters and setters::
	 */
	public PlayerSprite getPSprite() {
		return pSprite;
	}
	public void setPSprite(PlayerSprite sprite) {
		pSprite = sprite;
	}
	public int getWalkingSpeed() {
		return walkingSpeed;
	}
	public void setWalkingSpeed(int walkingSpeed) {
		this.walkingSpeed = walkingSpeed;
	}
	public boolean isJumping() {
		return jumping;
	}
	public void setJumping(boolean jumping) {
		this.jumping = jumping;
	}
	public boolean isFalling() {
		return falling;
	}
	public void setFalling(boolean falling) {
		this.falling = falling;
	}
	public int getJumpingSpeed() {
		return jumpingSpeed;
	}
	public void setJumpingSpeed(int jumpingSpeed) {
		this.jumpingSpeed = jumpingSpeed;
	}
	public Vector2D getVPos() {
		return vPos;
	}
	public void setVPos(Vector2D pos) {
		vPos.setVector(pos);
	}
	public Vector2D getVSpeed() {
		return vSpeed;
	}
	public void setVSpeed(Vector2D speed) {
		vSpeed.setVector(speed);
	}

	public boolean isClubbed() {
		return clubbed;
	}

	public void setClubbed(boolean clubbed) {
		this.clubbed = clubbed;
	}

	public Club getClub() {
		return club;
	}

	public boolean isRight() {
		return right;
	}

	public void setRight(boolean right) {
		this.right = right;
	}

	public int getID() {
		return ID;
	}
}
