/*
 * TestClient.java
 *
 * Created on 18. november 2007, 16:54
 *
 * To change this template, choose Tools | Template Manager
 * and open the template in the editor.
 */

package pp.network;

import java.io.*;
import javax.microedition.io.*;
import java.util.Hashtable;

/**
 * Class that sends and receives information to the YAR game server.
 */
public class ClientConnector{
    
    // I/O objects and socket::
     private SocketConnection socket = null;
     private OutputStream outStream = null;
     private OutputStreamWriter out = null;
     private InputStream inStream=null;
     private InputStreamReader in=null;
     
     // Socket streams::
     private Sender sender;
     private Receiver receiver;
     
     //Variables::
     private MessageHandler mHandler;
    
     /**
      * Recieves the address and port to the YAR game server and creates the 
      * necessary streams and sockets.
      */
     public ClientConnector(String adress,int port,MessageHandler m){
         try {
			 mHandler=m;
			 //creates the socket that connects to the YAR server::
			 socket = (SocketConnection)Connector.open("socket://"+adress+":"+port);
			
			 //Creates inout and output streams on the socket::
			 outStream = socket.openOutputStream();
			 out=new OutputStreamWriter(outStream);
			 inStream = socket.openInputStream();
			 in=new InputStreamReader(inStream);
			
			 //Creates sender and reseiver threads::
			 sender=new Sender();
			 receiver=new Receiver();
			 new Thread(sender).start();
			 new Thread(receiver).start();
        } catch (Exception e){
        	e.printStackTrace();
        }
     }
     
     /**
      * Receives a message as a string and uses the sender thread to send
      * the message to the YAR game server.
      */
     public void sendMessage(String message){
         sender.sendMessage(message);
     }
     
     /**
      * Closes the various streams used by the client connector
      */
     public void close(){
         try{
        	 //Disconnect the send/receive threads::
        	 receiver.disconnect();
        	 sender.disconnect();
        	 
        	 //Close the I/O streams::
             out.close();
             in.close();
             socket.close();
         }catch(Exception e){
             e.printStackTrace();
         }
     }
     
     /**
      * Sets the message handler
      */
     public void setMessageHandler(MessageHandler mh){
         mHandler=mh;
     }
     
     /**
      * A private class that receives messages from the server and passes them to the 
      * message handler
      */
    private class Receiver implements Runnable{
        
    	//Variables::
        private boolean running=true;
        
        /*
         * Stops the receiver loop
         */
        public void disconnect(){
        	running=false;
        }
        
        /**
         * Receiver loop that constantly checks for new chars to read from the 
         * socket connection stream. When theres is a new String of information
         * send to the receiver, the receiver reads the string as a string of chars,
         * and then notifies about the incomming message.
         */
        public void run() {
            while(socket==null){/*wait*/}
            while(running){
                StringBuffer message = new StringBuffer();
                try{                    
                    //Read message::
                    while(in.ready()) {
                        char character = (char)in.read();
                        if (character == '|') {
                            //System.out.println("M:"+message.toString());
                            mHandler.handleMessage(message.toString());
                            message.delete(0,message.length());// = new StringBuffer();
                        }
                        else {
                        	//Ignore line break and tab characters::
                        	if(character!='\n' && character !='\r'){
                        		message.append(character);
                            }
                        }
                    }                    
                    Thread.sleep(10);
                }catch(Exception e){
                	e.printStackTrace();
                }
            }            
        }
    
    }
    
    /**
     * A private class that sends messages to the game server
     */
    private class Sender implements Runnable{
        
    	//Variables::
    	private Hashtable messageQueue;
        boolean running=true;
        
        //Constructor::
        public Sender(){
            messageQueue=new Hashtable();
        }
        
        /**
         * Receives a message to send, adds the mesage to the message queue, and 
         * then notifies the sender loop that there is a message ready to be sent
         */
        public void sendMessage(String message){
            messageQueue.put(message,message); //.add(message);
            synchronized(this){
                this.notify();
            }
        }
        
        /*
         * Stops the sender loop
         */
        public void disconnect(){
        	running=false;
        }
        
        /**
         * The sender loop which sends all messages in the messagequeue, but sleeps
         * and waits for additional messages to send when the messagequeue has been 
         * emptied.
         */
        public void run() {
            while(running){
            	
                /* 
                 * While the message queue is not empty, try to send the messages to the server
                 * When the message queue is emptied the thread sleeps until notified about 
                 * new messages
                 * 
                 */
                synchronized(this){
                    try{
                        //While the messagequeue still has messages, get each message and send the
                        //messages to the server.
                        while(messageQueue.elements().hasMoreElements()){
                            String m=(String)messageQueue.elements().nextElement();
                            out.write(m+"\n");
                            messageQueue.remove(m);
                            //System.out.println("Sent message: "+m);
                            Thread.sleep(5);
                        }
                        this.wait();
                    }catch(Exception e){
                        e.printStackTrace();
                    }
                }
            }
        }
    }    
}
