package pp.ui;

import java.io.IOException;
import java.util.Enumeration;
import java.util.Hashtable;
import java.util.Vector;
import javax.microedition.lcdui.*;
import javax.microedition.lcdui.game.*;
import javax.microedition.midlet.*;
import pp.model.*;
import pp.model.event.*;
import pp.network.*;

public class GameWindow extends GameCanvas implements Runnable,CommandListener,MessageListener{

	//Variables::
	private MIDlet parentMID;
	private boolean gamePause;
	private boolean running;
	private LayerManager layerManager;
	private Vector2D viewPoint;//Temp variables(used for testing movement)
	private int width,height;
	private Command exitC;//Exit command
	private Command helpC;//Help command
	private Player player;//Main player
	private Hashtable playerList;//Other players connected through network
	private LevelGenerator lvlGenerator;//Creates levels
	private PPTrigger trigger;
	private Level lvl;	
	private double dt=0;long t=System.currentTimeMillis();//Time variables
	private boolean leftPressed=false,rightPressed=false;//Key press booleans
	private MessageHandler messageHandler;
	private ClientConnector connector;
	private boolean exit;
	
	//Constants::
	private final double GRAVITY=395.0;
	private final double UNIT_CONVERSION=1;
	private final double MAX_TIME_STEP=0.05;
	
	//Constructor::
	public GameWindow(MIDlet parentMID,ClientConnector cc,Player p) throws IOException{
		super(true);
		this.parentMID=parentMID;
		gamePause=false;
		running=true;
		layerManager=new LayerManager();
		playerList=new Hashtable();		
		
		//Initilializes commands::
		exitC=new Command("Exit",Command.EXIT,0);
		//helpC=new Command("Help",Command.HELP,0);
		//addCommand(helpC);
		addCommand(exitC);
		setCommandListener(this);
		this.setFullScreenMode(true);
		
		//Initialize levels::
		viewPoint=new Vector2D(0,0);
		trigger=new PPTrigger();
		width=getWidth();
		height=getHeight();
		lvlGenerator=new LevelGenerator();	
		player=p;
		layerManager.append(player.getClub());
		layerManager.append(player.getPSprite());
		//loadLevel(LevelGenerator.LEVEL_1_NAME, LevelGenerator.LEVEL_1_SCRIPT_NAME);
		
		//Network initialization::
		messageHandler=new MessageHandler(this);
		connector=cc;
		connector.setMessageHandler(messageHandler);
	}
	//Main game loop::
	public void run() {
		//Get the Graphics object for the off-screen buffer
        Graphics g = getGraphics();        
        int timeStep=100;
        while (running) {
            long start=System.currentTimeMillis();
            while(!gamePause){
                try{
                	if(lvl!=null){
		            	//Check player states::
		            	checkPlayer();
		    
		            	//Check for input::
		            	input();
		            	
		            	//Render the world::
		            	render(g);
                	}
                }catch(Exception e){
                	e.printStackTrace();
                }
            }            
            
            //Find out how long the loop round execution took:
            long end=System.currentTimeMillis();
            int time=(int)(end-start);

            //Sleep between each iteration, try to maintain a constant framerate:
            if(time<timeStep){
                try{
                    Thread.sleep(time-timeStep);
                }catch(Exception e){}
            }
        }
	}
	//Check Player states::
	public void checkPlayer(){
		//Updates the time::
		long tempT=System.currentTimeMillis();
		dt=(tempT-t)/1000.0;
		if(dt>this.MAX_TIME_STEP)dt=this.MAX_TIME_STEP;
		t=tempT;		
		Vector2D vSpeed=player.getVSpeed();//Speed vector
		
		//Check for collisions::		
		//collision();
		
		//Compute real movement relative to the passed time between each round in the game loop::
		int xx=(int)(UNIT_CONVERSION*(vSpeed.x*dt));
		int yy=(int)(UNIT_CONVERSION*(vSpeed.y*dt));
		
		//If the player is jumping or falling, update the y position of the player::
		if(player.isJumping() || player.isFalling() || player.isClubbed()){				
			vSpeed.y+=GRAVITY*dt;				
		}
		
		/*
		 * If the player has been clubbed, move the player regardless of keypresses, otherwise 
		 * compare keypresses::
		 */
		if(!player.isClubbed()){
			//No sliding, i.e. sudden stop when the player releases the button::
			if(xx<0 && !leftPressed){
				vSpeed.x=0;
				xx=0;
			}
			//No sliding, i.e. sudden stop when the player releases the button::
			if(xx>0 && !rightPressed){
				vSpeed.x=0;
				xx=0;
			}
		}else{
			if(vSpeed.x==0 && vSpeed.y==0){
				player.setClubbed(false);
			}
		}
		
		//Update player position::
		player.getVPos().addVectorValues(xx,yy);
		player.getClub().addVectorValues(xx, yy);
		viewPoint.addVectorValues(xx, yy);		
		player.updateSpritePos();
		
		//Collisions::
		collision();
		if(lvl!=null)clubCollision();
		if(lvl!=null)exitCollision();
	}
	//Renders the game world::
	public void render(Graphics g){
		if(lvl!=null){
			g.setColor(255, 255, 255);
			g.fillRect(0, 0, width, height);
			layerManager.setViewWindow(viewPoint.x, viewPoint.y, width, height);
			layerManager.paint(g,0,0);
			flushGraphics();
		}
	}
	//KOLLISJONSTEST LAGET MED HJELP AV TORO(funker ikke 100%)::
	public boolean collision2(){
		for(int i=0;i<lvl.getLvlObjectSize();i++){
			PPObject obj=lvl.getLvlObject(i);
			if(player.getPSprite().collidesWith(obj.getObjectSprite(), false) && obj.getObjectProperty().isHasCollision()){
				Vector2D vs=player.getVSpeed();//Player speed
				Vector2D speedOld=new Vector2D(vs.x,vs.y);
				Vector2D vp=player.getVPos();//Player position
				Vector2D vop=obj.getVPos();//Object position
				int width=LevelGenerator.LEVEL_BLOCK_WIDTHxHEIGHT;
				Vector2D v1=new Vector2D(0,0);
				Vector2D v2=new Vector2D(0,0);
				System.out.println("FART 1:::"+vs);
				
				//Find the corners that are involved in the collisions::
				//Upper left
				if(vp.x>vop.x && vp.x<vop.x+width && vp.y>vop.y && vp.y<vop.y+width){
					v1.setVectorValues(vp.x, vp.y);
					v2.setVectorValues(vop.x+width, vop.y+width);
				}
				//Upper right
				if(vp.x+width>vop.x && vp.x+width<vop.x+width && vp.y>vop.y && vp.y<vop.y+width){
					v1.setVectorValues(vp.x+width, vp.y);
					v2.setVectorValues(vop.x, vop.y+width);
				}
				//Lower right
				if(vp.x+width>vop.x && vp.x+width<vop.x+width && vp.y+width>vop.y && vp.y+width<vop.y+width){
					v1.setVectorValues(vp.x+width, vp.y+width);
					v2.setVectorValues(vop.x, vop.y);
				}
				//Lower left
				if(vp.x>vop.x && vp.x<vop.x+width && vp.y+width>vop.y && vp.y+width<vop.y+width){
					v1.setVectorValues(vp.x, vp.y+width);
					v2.setVectorValues(vop.x+width, vop.y);
				}
				
				//Regn ut kollisjonspunkter::
				Vector2D v3=new Vector2D(v2.x-v1.x,v2.y-v1.y);//v2-v1
				Vector2D v4=new Vector2D(0,0);
				
				//Mystix formel::
				if(vs.y>vs.x){
					if(vs.y==0){
						v4.x=v1.x;
					}else{
						v4.x=v3.y*vs.x/vs.y;//+v1.x;
					}
					v4.y=v3.y;
					vs.y=0;
				}else{
					v4.x=v3.x;
					if(vs.x==0){
						v4.y=v1.y;
					}else{
						v4.y=v3.x*vs.y/vs.x; //+v1.y;						
					}
					vs.x=0;
				}
				System.out.println("DISP:::X:"+v4.x+";Y:"+v4.y);
				System.out.println("FART 2:::"+vs);
				System.out.println("SPELLER:::"+vp);
				System.out.println("OBJEKT:::"+vop);
				vp.addVectorValues(v4.x,v4.y);
				player.getClub().addVectorValues(v4.x, v4.y);
				player.updateSpritePos();
				viewPoint.addVectorValues(v4.x,v4.y);
				System.out.println("Ny pos::::"+vp+"\n");
				
			}
		}		
		return false;
	}
	//Collision check::
	public boolean collision(){	
		if(lvl!=null){
			boolean coll=false;
			for(int i=0;i<lvl.getLvlObjectSize();i++){
				PPObject obj=lvl.getLvlObject(i);
				int w1=0,h1=0;
				if(player.getPSprite().collidesWith(obj.getObjectSprite(), false) && obj.getObjectProperty().isHasCollision()){
					int width=LevelGenerator.LEVEL_BLOCK_WIDTHxHEIGHT;//Standard width for all objects in the game
					Vector2D vs=player.getVSpeed();//Player speed
					Vector2D vp=player.getVPos();//Player position
					Vector2D vop=obj.getVPos();//Object position
					int w_1_2=(width/2);//Half of the width
					Vector2D vpc=new Vector2D(vp.x+(w_1_2),vp.y-(w_1_2));//centre of player
					Vector2D voc=new Vector2D(vop.x+w_1_2,vop.y-w_1_2);//centre of object
					Vector2D vpcMINvoc=new Vector2D(vpc.x-voc.x,vpc.y-voc.y);
					
					//Checks for collisions along the x-axis::
					//The player is moving to the left::
					if(vs.x<0){	
						//The plaeyr is falling::
						if(vs.y>0){
							//Checks to see which corners are inside the object::
							boolean ul=false,ll=false;
							//Upper left corner is inside
							if((vp.x>=vop.x && vp.x<=vop.x+width && vp.y>=vop.y && vp.y<=vop.y+width)){
								ul=true;							
							}
							//Lower left corner is inside::
							if(vp.x>=vop.x && vp.x<=vop.x+width && vp.y+width>=vop.y && vp.y+width<=vop.y+width){
								ll=true;
							}
							
							//Move the player in relation to which corners are inside the colliding object::
							if(ul && ll){
								//Both upper- and lower- right corners are inside the object::
								int w2=(vp.x-vop.x);
								w1=width-w2;
								vs.x=0;
							}else if(ul){
								//Upper left corner is inside the object::
								int w2=(vp.x-vop.x);
								w1=width-w2;
								vs.x=0;
							}
							else if(ll){
								//Lower left corner is inside the object::
								if((vpcMINvoc.x>0 && vpcMINvoc.y>0) || (vpcMINvoc.x<0 && vpcMINvoc.y>0)){
									int w2=(vp.x-vop.x);
									w1=width-w2;
									vs.x=0;
								}else if(vpcMINvoc.x>0 && vpcMINvoc.y<0){
									/*int w2=(vp.x-vop.x);
									w1=width-w2;
									vs.x=0;*/
								}
							}
						}
						//The player is not falling
						else{
							if(vp.x<(vop.x +width) && (vp.x+width)>(vop.x +width)){
								int w2=(vp.x-vop.x);
								w1=width-w2;
								vs.x=0;
							}
						}
					}
					//The player is moving to the right::
					else if(vs.x>0){
						/*
						//The player is falling::
						if(vs.y<0){
							//Checks to see which corners are inside the object::
							boolean ur=false,lr=false;
							//Upper right
							if(vp.x+width>=vop.x && vp.x+width<=vop.x+width && vp.y>=vop.y && vp.y<=vop.y+width){
								ur=true;
							}
							//Lower right
							if(vp.x+width>=vop.x && vp.x+width<=vop.x+width && vp.y+width>=vop.y && vp.y+width<=vop.y+width){
								lr=true;
							}
							
							//Move the player in relation to which corners are inside the colliding object::
							if(ur && lr){
								//Both upper- and lower- right corners are inside the object::
								int w2=((vop.x+width)-(vp.x+width));
								w1=-(width-w2);
								vs.x=0;
							}else if(ur){
								//Upper right corner is inside the object::
								int w2=((vop.x+width)-(vp.x+width));
								w1=-(width-w2);
								vs.x=0;
							}else if(lr){
								//Lower right corner is inside the object::
								if((vpcMINvoc.x>0 && vpcMINvoc.y>0) || (vpcMINvoc.x<0 && vpcMINvoc.y>0)){
									int w2=((vop.x+width)-(vp.x+width));
									w1=-(width-w2);
									vs.x=0;
								}else if(vpcMINvoc.x<0 && vpcMINvoc.y<0){
									/*int w2=(vp.x-vop.x);
									w1=width-w2;
									vs.x=0;*/
								//}
							//}
						//}
						//The player is not falling::
						//else{
							
							if(vp.x<vop.x && (vp.x+width)>vop.x){
								int w2=((vop.x+width)-(vp.x+width));
								w1=-(width-w2);
								vs.x=0;
							}
						//}				
					}
					
					//Checks for collisions along the y-axis::
					//The players i jumping::
					if(vs.y<0){
						if(vp.y<(vop.y+width) && (vp.y+width)>(vop.y+width)){
							player.setJumping(false);
							player.setFalling(true);
							if(player.isClubbed()){
								player.setClubbed(false);
							}
							vs.y=0;
							int h2=(vp.y+width)-(vop.y+width);
							h1=width-h2;
						}					
					}
					//the players is falling::
					else if(vs.y>0){
						if( vp.y<vop.y && (vp.y+width)>vop.y ){
							vs.y=0;
							player.setFalling(false);
							player.setJumping(false);
							
							//Checks to see whether the player has been clubbed::
							if(player.isClubbed()){
								player.setClubbed(false);
							}
							h1=-(width-(vop.y-vp.y));
						}					
					}
					player.getVPos().addVectorValues(w1, h1);
					player.getClub().addVectorValues(w1, h1);
					viewPoint.addVectorValues(w1,h1);
					player.updateSpritePos();
					//return true;
					coll=true;
				}
			}	
			if(!player.isJumping() && !player.isFalling() && !player.isClubbed() && !coll)player.setFalling(true);
			return false;
			}
		return false;
	}
	
	/*
	 * Checks to see whether the player is in the proximity of a trigger. The method returns the 
	 * trigger object if the player is in the proximity of a trigger, or null if the player
	 * is not.
	 */
	public PPObject triggerCollision(){
		for(int i=0;i<lvl.getTriggerObjectSize();i++){
			PPObject obj=lvl.getTriggerObject(i);
			if(player.getPSprite().collidesWith(obj.getObjectSprite(), false)){
				return obj;
			}
		}
		return null;
	}
	
	/*
	 * Checks to see whether the club of the player hits any of the other players, and if
	 * that is the case the player must send a message to the server such that the other 
	 * players can be notified and initiate the clubbing mechanic on themselves. 
	 */
	public void clubCollision(){
		String mBuf="";
		
		//Goes through the player list and checks to see whether they are clubbed::
		for(Enumeration e=playerList.elements();e.hasMoreElements();){
			Player p=(Player)e.nextElement();
			
			//Checks for a collision between the client player's club and the other players::
			if(player.clubPlayerCollision(p)){
				//Update the message buffer::
				mBuf+=messageHandler.createClubRequest(p.getID(),player.isRight());
			}
		}
		
		//When all collisions has been detected, send a message to the server about the collisions::
		if(!mBuf.equals("")){
			connector.sendMessage(mBuf);
		}
	}
	
	/*
	 * Checks to see whether the player collides with an exit object, which should lead
	 * to the client sending a notification to the server about the event.
	 */
	public void exitCollision(){
		if(!exit){
			//Check each exit object to see whether the player collides with them::
			for(int i=0;i<lvl.getExitObjectSize();i++){
				PPObject obj=lvl.getExitObject(i);
				if(player.getPSprite().collidesWith(obj.getObjectSprite(), false)){
					//If the player has collided, send a message to the server::
					connector.sendMessage(messageHandler.createExitReached());
					exit=true;
					return;
				}
			}
		}
	}
	
	/*
	 * Loads a level and its corresponding script file while resetting the layer manager and
	 * the current level if the current level is not null. 
	 */
	public void loadLevel(String name,String scriptName,Vector2D startPos){
		Level lvlT=this.lvlGenerator.readLevel(name, scriptName, trigger);
		
		//Removes the objects of the previus level from the layerManager::
		if(lvl!=null){
			for(int i=0;i<lvl.getLvlObjectSize();i++){
				layerManager.remove(lvl.getLvlObject(i).getObjectSprite());
			}			
		}
		lvl=lvlT;
		
		//Loads the objects in the level into the layer manager, and places the player::		
		//Adds each of the objects in the level to the layer manager::
		for(int i=0;i<lvl.getLvlObjectSize();i++)
		{
			PPObject obj=lvl.getLvlObject(i);
			layerManager.append(obj.getObjectSprite());			
		}
		
		//Adds the starting point of the player and updates the camera and player::
		player.setVPos(startPos);
		player.getClub().updateOwnerClub();
		
		//Centers the "camera" around the player::
		int kX=startPos.x+(LevelGenerator.LEVEL_BLOCK_WIDTHxHEIGHT/2);
		int kY=startPos.y+(LevelGenerator.LEVEL_BLOCK_WIDTHxHEIGHT/2);
		viewPoint.setVector(new Vector2D(kX-(getWidth()/2) , kY-(getHeight()/2)));
		exit=false;
		
	}
	
	//Checks the input::
	public void input(){
		//TODO:: create a button that check the triggers and fires an event when pressed
		int keyState = getKeyStates();
        try{
        	//Go left::
        	if ((keyState & LEFT_PRESSED) != 0){
        		leftPressed=true;
        		player.moveLeft(1);
        	}else{
        		leftPressed=false;
        	}
        	//Go right::
        	if ((keyState & RIGHT_PRESSED) != 0){
        		rightPressed=true;
        		player.moveRight(1);
        	}else{
        		rightPressed=false;
        	}
        	//Try to go up::
        	if ((keyState & UP_PRESSED) != 0){
        		
        	}
        	//Try to go down::
        	if ((keyState & DOWN_PRESSED) != 0){
        		
        	}        	 
        	//Jump::
        	if( (keyState & FIRE_PRESSED) != 0 ){
        		player.jump();
        	}
        	//Try to trigger an event::
        	if( (keyState & GAME_A_PRESSED) !=0 ){
        		PPObject obj;
        		if( (obj=triggerCollision()) != null ){
        			//If a collision with a trigger is detected, send a notice to the servera bout it::
        			connector.sendMessage(messageHandler.createTriggerEventRequest(obj.getObjectProperty().getTriggerID()));
        		}
        	}
        	//Try to initiate clubbing::
        	if( (keyState & GAME_B_PRESSED) !=0 ){
        		player.getClub().startClubbing();
        	}
        }catch(Exception e){
        	e.printStackTrace();
        }
	}
	
	//Controlls the commands::
	public void commandAction(Command command, Displayable arg1) {
		if(command==exitC){
			parentMID.notifyDestroyed();
		}
		if(command==helpC){
			//HelpWindow hw=new HelpWindow(parentMID,this);
		}
	}
	
    //Starts the game loop:
    public void showNotify(){
        new Thread(this).start();
    }   
    
    //Adds a player to the game::
    public void addPlayer(Player p){
    	addPlayerToPlayerList(p);
    	layerManager.append(p.getClub());
    	layerManager.append(p.getPSprite());    	
    }
    
    //Remove player from the game::
    public void removePlayer(int id){
    	Player p=(Player)playerList.remove(new Integer(id));
    	layerManager.remove(p.getPSprite());
    	layerManager.remove(p.getClub());
    	p=null;
    }
    
    //Updates the position of the player that relates to 'id'::
    public void updatePlayerPos(int id,Vector2D pos){
    	Player p=(Player)playerList.get(new Integer(id));    	
    	p.setVPos(pos);
    	p.getClub().updateOwnerClub();
    	p.updateSpritePos();
    }
    
    //Adds a new player to the playerlist::
    private void addPlayerToPlayerList(Player p){
    	playerList.put(new Integer(p.getID()), p);
    }
    
    //Gets the playerlist size::
    public int getPlayerListSize(){
    	return playerList.size();
    }
    
    //Gets a player from the playerlist::
    private Player getPlayerFromPlayerList(int id){
    	return (Player)playerList.get(new Integer(id));
    }
    
    //Removes a player from the playerlist::
    private void removePlayerFromPlayerList(int id){
    	playerList.remove(new Integer(id));
    }
    
    //MESSAGE LISTENER METHODS::
    //SERVER METHOD::
	public void notifyClubRequest(int id,boolean right) {		
	}
	
	/*
	 * The server has sent a message to this player notifying the player that a new player has
	 * joined the game 
	 */
	public void notifyCreatePlayer(int id, Vector2D pos) {
		try {
			addPlayer(new Player(pos,id));
		} catch (IOException e) {
			e.printStackTrace();
		}		
	}
	
	//SERVER METHOD::
	public void notifyDisconnectRequest() {
	}
	//SERVER METHOD::
	public void notifyExitReached() {		
	}
	
	/*
	 * The server has sent a message that this player should initate the clubbing mechanism
	 */
	public void notifyInitClubbing(boolean right) {
		player.clubMe(right);
	}
	
	//SERVER METHOD::
	public void notifyJoinAck(int id) {
		//PROBABLY NOT USED HERE, BUT IN THE PLATFORMPUZZLER CLASS
	}
	//SERVER METHOD::
	public void notifyJoinRequest() {
	}
	
	/*
	 * The server has sent a message that a new level should be loaded
	 */
	public void notifyLoadLevel(String lvlName,Vector2D startPos) {
		//Check to see whether the lvlName paramter indicates that there are no mroe levels::
		if(lvlName.equals(Level.LEVEL_END)){
			//Ends the game with the end window::
			running=false;
			connector.sendMessage(messageHandler.createDisconnectRequest());
			connector.close();	
			EndWindow w=new EndWindow(parentMID);					
		}else{
			//Loads the new level::
			loadLevel(lvlName+".lvl", lvlName+".slvl",startPos);
		}
	}
	
	/*
	 * The server has sent a message saying that a player has disconnected, and thus 
	 * that player should be removed from the game world
	 */
	public void notifyPlayerDisconnect(int id) {
		removePlayer(id);
	}
	
	//SERVER METHOD::
	public void notifyPosition(Vector2D pos) {
	}
	
	/*
	 * The server has sent a message saying that a player has moved, and should thus be moved
	 * in the game world.
	 */
	public void notifyPositionChange(int id, Vector2D pos) {
		updatePlayerPos(id, pos);
	}
	
	/*
	 * The server has sent a message that requests the client to send his position in the game
	 * world back to the server.
	 */
	public void notifyPositionRequest() {
		connector.sendMessage(messageHandler.createPosition(player.getVPos()));
	}
	
	/*
	 * The server has sent the client a message informing the client that a new event should
	 * be triggered.
	 */
	public void notifyTriggerEvent(int eventID) {
		PPObject obj=null;
		for(int i=0;i<lvl.getTriggerObjectSize();i++){
			if(lvl.getTriggerObject(i).getObjectProperty().getTriggerID()==eventID){
				obj=lvl.getTriggerObject(i);
				break;
			}
		}
		trigger.firePPEvent(new PPEvent(obj,eventID));
	}
	
	//SERVER METHOD
	public void notifyTriggerEventRequest(int eventID) {		
	}
}