package pps.model;

import java.io.BufferedReader;
import java.io.File;
import java.io.FileReader;
import java.io.InputStream;
import java.util.Hashtable;
import java.util.Vector;

public class LevelCoordinator {
	
	//Variables::
	private Vector levelNameList;
	private Vector<Vector2D> startPoints;
	private int startPointCounter=0;
	private String currentLevel;
	private int currentLevelInt=0;
	
	//Constants::
	private static final String LEVEL_NAME_SUFFIX=".lvl";
	private static final String LEVEL_SCRIPT_NAME_SUFFIX=".slvl";
	public static final String LEVEL_END="none";
	private final String DATA_FILE_SUFFIX=".dta";
	private final String DATA_LEVEL_FILE="lvlnames";
	private final String DATA_LEVEL_FILE_LOCATION="pps/data/";
	private final char LEVEL_CONST_START_BLOCK='s';
	private final char LEVEL_CONST_END_BLOCK='e';
	private final char LEVEL_CONST_SPACE='\u0020';
	private final char LEVEL_CONST_LINE_BREAK='\n';
	private final char LEVEL_CONST_INVISIBLE_SPACE_THINGIE='\r';	
	//Constructor::
	public LevelCoordinator(){
		levelNameList=new Vector();
		startPoints=new Vector<Vector2D>();
		readLevelFile();
		currentLevel=(String)levelNameList.get(0);
		currentLevelInt=1;
		updateStartPositions();
	}
	
	/**
	 * Returns the current level of the server
	 * 
	 */
	public String getCurrentLevel(){
		return currentLevel;
	}
	
	/**
	 * Returns a starting point for the player. The starting point will not be equal for
	 * two consequetive calls if there are more than 2 starting points in the level
	 */
	public Vector2D getStartingPoint(){
		//Check to see whether there are any starting points::
		if(!startPoints.isEmpty()){
			//Returns the startpoint at the current counter position::
			Vector2D ret=startPoints.elementAt(startPointCounter);
			
			//Update the counter position::
			startPointCounter=( (++startPointCounter) % startPoints.size());
			
			//Return position::
			return ret;
		}
		return null;
	}
	
	/**
	 * Changes the level to the next level, and returns the new level name
	 */
	public String nextLevel(){
		if(currentLevelInt>=levelNameList.size()){
			currentLevel=LEVEL_END;
			return currentLevel;
		}else{
			currentLevel=(String)levelNameList.get(currentLevelInt);
			currentLevelInt++;
			updateStartPositions();
			return currentLevel;
		}		
	}
	
	/**
	 * Reads the names of the levels available in the pp game into the 
	 * name array.
	 *
	 */
	public void readLevelFile(){
		File file = new File(DATA_LEVEL_FILE_LOCATION+DATA_LEVEL_FILE+DATA_FILE_SUFFIX);
		FileReader fis = null;
		BufferedReader dis = null;
		
		try {
			fis = new FileReader(file);
			dis = new BufferedReader(fis);
			
			String line="";
			
			//Read the file into the array::
			while ( (line=dis.readLine())!=null ) {
				levelNameList.add(line);
			}
			
			//Close the streams::
			fis.close();
			dis.close();
		} catch (Exception e) {
			e.printStackTrace();
		}
	}
	
	/*
	 * Private method that updates the players starting  position list for the 
	 * current level
	 */
	private void updateStartPositions(){
		InputStream is = getClass().getResourceAsStream("/pps/data/"+currentLevel+LEVEL_NAME_SUFFIX);//The level file
		try 
	    {
	    	//Temp level, returned if the level creation is succesful:: 	
			//File Buffers::
			StringBuffer sb = new StringBuffer();
			int chr, i = 0;
			
			// Read the level information until the end of the stream::      
			while ((chr = is.read()) != -1){
			    sb.append((char) chr);   
			}
			
			//Goes through the level buffer creating level objects::
			i=0;		
			//Reads the rest of the level::
			int XX=0,YY=0;//hBlocks, use to calculate x and y position of the objects;
			while(i<sb.length()){
				char c=sb.charAt(i);
				//Checks each char to see whether a object should be created::
				//SPACE,white block::
				if(c==LEVEL_CONST_SPACE){
					XX++;
				}
				//'\r', do nothing::
				else if(c==LEVEL_CONST_INVISIBLE_SPACE_THINGIE){
				}
				//LINE-BREAK, update variables::
				else if(c==LEVEL_CONST_LINE_BREAK){
					XX=0;//Set the x position to 0 since a new row of the map has started
					YY++;//Updates the  position since a new row has started
				}
				//Create a start object::
				else if(c==LEVEL_CONST_START_BLOCK){					
					Vector2D v=new Vector2D(XX*32,YY*32);
					startPoints.add(v);
					XX++;
				}
				//Create exit block::
				else if(c==LEVEL_CONST_END_BLOCK){
					XX++;
				}
				
				//Update xx::
				else{					
					XX++;
				}
				i++;
			}
			
			//Close the input streams::
			is.close();			
	    }catch (Exception e){  
	    	e.printStackTrace();
		}
	}	
}
