/*
 * MessageHandler.java
 */

package pps.server;

import pps.model.Vector2D;

/**
 *
 * @author ivind
 */
public class MessageHandler {
    
    //Static constants::
    public final static String MSG_JOIN_REQUEST="jo";
    public final static String MSG_JOIN_ACK="ja";
    public final static String MSG_CREATE_PLAYER="cp";
    public final static String MSG_CLUB_REQUEST="cr";
    public final static String MSG_INIT_CLUBBING="ic";
    public final static String MSG_EXIT_REACHED="er";
    public final static String MSG_LOAD_LEVEL="ll";
    public final static String MSG_DISCONNECT_REQUEST="dr";
    public final static String MSG_PLAYER_DISCONNECT="pd";
    public final static String MSG_POSITION_REQUEST="pr";
    public final static String MSG_POSITION="po";
    public final static String MSG_PLAYER_POSITION_CHANGE="ppc";
    public final static String MSG_TRIGGER_EVENT_REQUEST="ter";
    public final static String MSG_TRIGGER_EVENT="te";
    
    //Variables::
    private MessageListener msgListener;
    
    /** Creates a new instance of MessageHandler. The class creates and handles messages
     * on the form:
     * <msg prefix>|
     * <msg prefix>:<msg info1>,<msg info2>,..,<msg info N>|
     * here the first message is typically a request, while the second message typically contains 
     * information which can range form 1 to N information.
     */
    public MessageHandler(MessageListener msg) {
        msgListener=msg;
    }
    
    //JOIN_REQUEST: <prefix>|
    public String createJoinRequest(){
    	return MSG_JOIN_REQUEST+":|";
    }
    //JOIN_ACK: <prefix>:<player id>|
    public String createJoinAck(int id){
    	return MSG_JOIN_ACK+":"+id+"|";
    }
    //CREATE_PLAYER: <prefix>:<player id>,<player x-pos>,<player y-pos>|
    public String createCreatePlayer(int id,Vector2D pos){
    	return MSG_CREATE_PLAYER+":"+id+","+pos.x+","+pos.y+"|";
    }
    //CLUB_REQUEST: <prefix>:<player id>|
    public String createClubRequest(int id,boolean right){
    	return MSG_CLUB_REQUEST+":"+id+","+right+"|";
    }
    //INIT_CLUBBING: <prefix>:<right/left>|
    public String createInitClubbing(boolean r){
    	return MSG_INIT_CLUBBING+":"+r+"|";
    }
    //EXIT_REACHED: <prefix>|
    public String createExitReached(){
    	return MSG_EXIT_REACHED+":|";
    }
    //LOAD_LEVEL: <prefix>:<level name>|
    public String createLoadLevel(String name,Vector2D startPos){
    	return MSG_LOAD_LEVEL+":"+name+","+startPos.x+","+startPos.y+"|";
    }
    //DISCONNECT_REQUEST: <prefix>|
    public String createDisconnectRequest(){
    	return MSG_DISCONNECT_REQUEST+":|";
    }
    //PLAYER_DISCONNECT: <prefix>:<player id>|
    public String createPlayerDisconnect(int id){
    	return MSG_PLAYER_DISCONNECT+":"+id+"|";
    }
    //POSITION_REQUEST: <prefix>|
    public String createPositionRequest(){
    	return MSG_POSITION_REQUEST+":|";
    }
    //POSITION: <prefix>:<player x-pos>,<player y-pos>|
    public String createPosition(Vector2D pos){
    	return MSG_POSITION+":"+pos.x+","+pos.y+"|";
    }
    //PLAYER_POSITION_CAHNGE: <prefix>:<player id>,<player x-pos>,<player y-pos>|
    public String createPlayerPositionChange(int id,Vector2D pos){
    	return MSG_PLAYER_POSITION_CHANGE+":"+id+","+pos.x+","+pos.y+"|";
    }
    //TRIGGER_EVEMT: <prefix>:<event id>|
    public String createTriggerEvent(int eventID){
    	return MSG_TRIGGER_EVENT+":"+eventID+"|";
    }
    //TRIGGER_EVENT_REQUEST: <prefix>:<event id>|
    public String createTriggerEventRequest(int eventID){
    	return MSG_TRIGGER_EVENT_REQUEST+":"+eventID+"|";
    }
    
    /**
     * Reads messages on the format under and calls the appropriate message in the
     * messagelistener which creates either an non-response action or a response action
     *
     * <msg>        := <msg_prefix>':'<msg_part_1>','<msg_part_2>','...','<msg_part_n>'|'
     * <msg_part_n> := int|boolean|String
     */
    public void handleMessage(String m,PlayerSession ps){ 
    	StringBuffer mbuf=new StringBuffer(m);//message buffer
    	
    	//Find the prefix::
    	String pref="";//Prefix string
    	char c;//Temp char
    	int i=0;
    	while( (c=mbuf.charAt(i))!=':' ){
    		//Append char to prefix string
    		pref+=c;
    		i++;
    	}    	
    	
    	//We now have the prefix so we must check the prefix and take appropriate actions::
    	//Check for a join request::
    	if(pref.equals(MSG_JOIN_REQUEST)){
    		msgListener.notifyJoinRequest(ps);
    		return;
    	}
    	//Check for a join accknowledgement:.
    	else if(pref.equals(MSG_JOIN_ACK)){
    		i++;//go past ':'
    		String sid="";
    		int id=0;
    		while(i<mbuf.length()){
    			sid+=mbuf.charAt(i);
    			i++;
    		}
    		//Parse the string to an integer::
    		id=Integer.parseInt(sid);
    		msgListener.notifyJoinAck(id,ps);
    		return;
    	}
    	//Check for a create player message::
    	else if(pref.equals(MSG_CREATE_PLAYER)){
    		i++;//go past ':'
    		Vector2D pos=new Vector2D(0,0);
    		int id;
    		String t="";
    		
    		//Find the id::
    		while( (c=mbuf.charAt(i)) != ',' ){
    			t+=c;
    			i++;
    		}
    		id=Integer.parseInt(t);
    		
    		//Find the x-pos::
    		t="";
    		i++;
    		while( (c=mbuf.charAt(i)) !=',' ){
    			 t+=c;
    			 i++;
    		}
    		pos.addVectorValues(Integer.parseInt(t), 0);
    		
    		//Find the y-pos::
    		t="";
    		i++;
    		while( i<mbuf.length() ){
    			t+=mbuf.charAt(i);
    			i++;
    		}
    		pos.addVectorValues(0, Integer.parseInt(t));
    		
    		//Notify::
    		msgListener.notifyCreatePlayer(id,pos,ps);
    		return;    		
    	}
    	//Check for a club request::
    	else if(pref.equals(MSG_CLUB_REQUEST)){
    		i++;//go past ':'
    		String sid="";
    		int id=0;
    		
    		while((c=mbuf.charAt(i)) != ','){
    			sid+=c;
    			i++;
    		}
    		//Parse the string to an integer::
    		id=Integer.parseInt(sid);
    		
    		//find the boolean::
    		i++;
    		String t="";
    		while(i<mbuf.length()){
    			t+=mbuf.charAt(i);
    			i++;
    		}
    		//Find the boolean value of 't'::
    		if(t.equals(Boolean.TRUE.toString())){
    			//The boolean value is 'true'::
    			msgListener.notifyClubRequest(id,true,ps);
    			return;
    		}else{
    			//the boolean value is 'false'::
    			msgListener.notifyClubRequest(id,false,ps);
    			return;
    		}
    	}
    	//Check for a init clubbing message::
    	else if(pref.equals(MSG_INIT_CLUBBING)){
    		i++;//go past ':'
    		String t="";
    		//find the boolean::
    		while(i<mbuf.length()){
    			t+=mbuf.charAt(i);
    			i++;
    		}
    		//Find the boolean value of 't'::
    		if(t.equals(Boolean.TRUE.toString())){
    			//The boolean value is 'true'::
    			msgListener.notifyInitClubbing(true,ps);
    			return;
    		}else{
    			//the boolean value is 'false'::
    			msgListener.notifyInitClubbing(false,ps);
    			return;
    		}
    	}
    	//Check for a exit reached message::
    	else if(pref.equals(MSG_EXIT_REACHED)){
    		msgListener.notifyExitReached(ps);
    		return;
    	}
    	//Check for a load level message::
    	else if(pref.equals(MSG_LOAD_LEVEL)){
    		i++;//go past ':'
    		String lvl="";
    		String t="";
    		Vector2D pos=new Vector2D(0,0);
    		
    		//Find the level name, which is a string::
    		while((c=mbuf.charAt(i)) !=',' ){
    			lvl+=mbuf.charAt(i);
    			i++;
    		}
    		
    		//Find the x-pos::
    		t="";
    		i++;
    		while( (c=mbuf.charAt(i)) !=',' ){
    			 t+=c;
    			 i++;
    		}
    		pos.addVectorValues(Integer.parseInt(t), 0);
    		
    		//Find the y-pos::
    		t="";
    		i++;
    		while( i<mbuf.length() ){
    			t+=mbuf.charAt(i);
    			i++;
    		}
    		pos.addVectorValues(0, Integer.parseInt(t));
    		
    		//Notify::
    		msgListener.notifyLoadLevel(lvl,pos,ps);
    		return;
    	}
    	//Check for a disconnect request::
    	else if(pref.equals(MSG_DISCONNECT_REQUEST)){
    		msgListener.notifyDisconnectRequest(ps);
    		return;
    	}
    	//Check for a player disconnect message::
    	else if(pref.equals(MSG_PLAYER_DISCONNECT)){
    		i++;//go past ':'
    		String sid="";
    		int id=0;
    		while(i<mbuf.length()){
    			sid+=mbuf.charAt(i);
    			i++;
    		}
    		//Parse the string to an integer::
    		id=Integer.parseInt(sid);
    		msgListener.notifyPlayerDisconnect(id,ps);
    		return;
    	}
    	//Check for a position request::
    	else if(pref.equals(MSG_POSITION_REQUEST)){
    		msgListener.notifyPositionRequest(ps);
    		return;
    	}
    	//Check for a position message:
    	else if(pref.equals(MSG_POSITION)){
    		i++;//go past ':'
    		Vector2D pos=new Vector2D(0,0);
    		String t="";
    		
    		//Find the x-pos::
    		while( (c=mbuf.charAt(i)) !=',' ){
    			 t+=c;
    			 i++;
    		}
    		pos.addVectorValues(Integer.parseInt(t), 0);
    		
    		//Find the y-pos::
    		t="";
    		i++;
    		while( i<mbuf.length() ){
    			t+=mbuf.charAt(i);
    			i++;
    		}
    		pos.addVectorValues(0, Integer.parseInt(t));
    		
    		//Notify::
    		msgListener.notifyPosition(pos,ps);
    		return;
    	}
    	//Check for a player position change message:
    	else if(pref.equals(MSG_PLAYER_POSITION_CHANGE)){
    		i++;//go past ':'
    		Vector2D pos=new Vector2D(0,0);
    		int id;
    		String t="";
    		
    		//Find the id::
    		while( (c=mbuf.charAt(i)) != ',' ){
    			t+=c;
    			i++;
    		}
    		id=Integer.parseInt(t);
    		
    		//Find the x-pos::
    		t="";
    		i++;
    		while( (c=mbuf.charAt(i)) !=',' ){
    			 t+=c;
    			 i++;
    		}
    		pos.addVectorValues(Integer.parseInt(t), 0);
    		
    		//Find the y-pos::
    		t="";
    		i++;
    		while( i<mbuf.length() ){
    			t+=mbuf.charAt(i);
    			i++;
    		}
    		pos.addVectorValues(0, Integer.parseInt(t));
    		
    		//Notify::
    		msgListener.notifyPositionChange(id, pos,ps);
    		return;
    	}
    	//Check for a trigger event::
    	else if(pref.equals(MSG_TRIGGER_EVENT)){
    		i++;//go past ':'
    		String sid="";
    		int eid=0;
    		while(i<mbuf.length()){
    			sid+=mbuf.charAt(i);
    			i++;
    		}
    		//Parse the string to an integer::
    		eid=Integer.parseInt(sid);
    		msgListener.notifyTriggerEvent(eid,ps);
    		return;
    	}
    	//Check for a trigger event request
    	else if(pref.equals(MSG_TRIGGER_EVENT_REQUEST)){
    		i++;//go past ':'
    		String sid="";
    		int eid=0;
    		while(i<mbuf.length()){
    			sid+=mbuf.charAt(i);
    			i++;
    		}
    		//Parse the string to an integer::
    		eid=Integer.parseInt(sid);
    		msgListener.notifyTriggerEventRequest(eid,ps);
    		return;
    	}
    } 
}