/*
 * MessageListener.java
 *
 */

package pps.server;

import pps.model.Vector2D;

/**
 * Interface that listens to incomming and outgoing messages
 *
 * @author ivind
 */
public interface MessageListener {
	//Position messages::
	public void notifyPositionRequest(PlayerSession ps);//Sent by the server requesting the position of the player
	public void notifyPosition(Vector2D pos,PlayerSession ps);//the response from a client to a server request
    public void notifyPositionChange(int id,Vector2D pos,PlayerSession ps);//notifies a client that another player has moved
    
    //Connection::
    public void notifyJoinRequest(PlayerSession ps);//A client sends a request to join the server
    public void notifyJoinAck(int id,PlayerSession ps);//The server answer the join request and gives the client an ID
    public void notifyCreatePlayer(int id,Vector2D pos,PlayerSession ps);//The server notifies a client that it should create a new player
    public void notifyDisconnectRequest(PlayerSession ps);//A client notifies the serer that it is going to disconnect
    public void notifyPlayerDisconnect(int id,PlayerSession ps);//The server notifies the rest of the clients that a client has disconnected
        
    //Special actions::
    public void notifyClubRequest(int id,boolean right,PlayerSession ps);//A player wants to club another player
    public void notifyInitClubbing(boolean right,PlayerSession ps);//the player notifies the other player that it should be clubbed
    public void notifyLoadLevel(String lvlName,Vector2D startPos,PlayerSession ps);//The server notifies the client that it should load the level
    public void notifyExitReached(PlayerSession ps);//A client notifies the server that it has reached the exit of the level
    public void notifyTriggerEventRequest(int eventID,PlayerSession ps);//The client sends a trigger request to the server
    public void notifyTriggerEvent(int eventID,PlayerSession ps);//The server sends a notice that a event should be triggered
}
